import { spawn } from 'child_process'
import path from 'path'
import R from 'ramda'

import md5 from '../lib/md5'
import { BundleConfig, CompileOutput } from '../matrix-compile/matrixCompile'

interface CliOptions {
  projectRoot: string
  metroConfigAbsPath: string
  bundleConfigs: BundleConfig[]
}

export default function compile({
  projectRoot,
  metroConfigAbsPath,
  bundleConfigs,
}: CliOptions): Promise<CompileOutput[]> {

  const promises: Promise<CompileOutput>[][] = bundleConfigs.map(({
    entryAbsPath,
    outputDir,
    getBundleName,
    platforms = ['ios', 'android'],
  }): Promise<CompileOutput>[] => {
    return platforms.map((platform): Promise<CompileOutput> => {
      const bundleName = getBundleName ? getBundleName(platform, entryAbsPath) : `${path.basename(entryAbsPath, '.js')}.${platform}.jsbundle`
      const bundleAbsPath = path.resolve(outputDir, bundleName)

      const bundleProcess = spawn(
        'node',
        [
          require.resolve('react-native/local-cli/cli.js', { paths: [projectRoot] }),
          'bundle',
          '--platform',
          platform,
          '--dev',
          'false',
          '--entry-file',
          entryAbsPath,
          '--bundle-output',
          bundleAbsPath,
          '--config',
          metroConfigAbsPath,
          '--assets-dest',
          // outputDir
          // ios打出assets, android可以公用
          platform === 'ios' ? outputDir : '',
          '--sourcemap-output',
          `${bundleAbsPath}.map`,
        ],
        {
          // 忽略输入, 把 stdout 输出到父进程的 stdout, 自己处理 stderr
          // stdio: ['ignore', 'inherit', 'pipe'],
          stdio: 'pipe',
          cwd: projectRoot,
        },
      )

      return new Promise((resolve, reject) => {
        let errOut: string[] = []

        bundleProcess.stderr.on('data', data => {
          errOut = errOut.concat(data.toString().split('\n'))
        })

        bundleProcess.on('close', async (code) => {
          if (code === 0) {
            try {
              const bundleMd5 = await md5(bundleAbsPath)
              resolve({
                entryAbsPath,
                bundleAbsPath,
                bundleMd5,
                platform,
              })
              return
            } catch (err) {
              reject(err)
            }
          }
          reject(errOut.join('\n'))
        })

        bundleProcess.on('error', (err) => {
          reject(`打包 RN 入口文件的进程无法启动, ${err}`)
        })
      })
    })
  })

  return Promise.all(R.flatten(promises))
}
